﻿using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Windows.Forms;


namespace Ruler
{
    public partial class FormConfig: Form
    {
		static bool changeinprogress = false;


        public FormConfig()
        {
            InitializeComponent();
        }


		public void UpdateFormConfig( )
		{
			Debug.Print( "UpdateFormConfig( )" );
			comboBoxLineColor.SelectedItem = GlobalSettings.PenColor.Name;
			checkBoxOpaqueBackground.Checked = GlobalSettings.OpaqueBackground;
			comboBoxBackgroundColor.SelectedItem = GlobalSettings.BackgroundColor.Name;
			comboBoxBackgroundColor.Enabled = GlobalSettings.OpaqueBackground;
			comboBoxPosition.SelectedIndex = ( GlobalSettings.Vertical ? 1 : 0 );
			if ( GlobalSettings.PositionHorizontal == -1 )
			{
				GlobalSettings.PositionHorizontal = GlobalSettings.HalfWidth;
			}
			if ( GlobalSettings.PositionVertical == -1 )
			{
				GlobalSettings.PositionVertical = GlobalSettings.HalfHeight;
			}
			if ( GlobalSettings.Vertical )
			{
				numericUpDownOffsetPixels.Maximum = GlobalSettings.ScreenWidthPx;
				GlobalSettings.Millimeter = (float)GlobalSettings.ScreenHeightPx / ( 10 * GlobalSettings.SelectedHeightCm );
				GlobalSettings.EighthOfAnInch = (float)GlobalSettings.ScreenHeightPx / ( 8 * GlobalSettings.ScreenHeightInch );
				if ( !changeinprogress )
				{
					numericUpDownOffsetPixels.Value = GlobalSettings.PositionHorizontal;
					numericUpDownOffsetPercentage.Value = (int)( 100 * GlobalSettings.PositionHorizontal / GlobalSettings.ScreenWidthPx );
				}
			}
			else
			{
				numericUpDownOffsetPixels.Maximum = GlobalSettings.ScreenHeightPx;
				GlobalSettings.Millimeter = (float)GlobalSettings.ScreenWidthPx / ( 10 * GlobalSettings.SelectedWidthCm );
				GlobalSettings.EighthOfAnInch = (float)GlobalSettings.ScreenWidthPx / ( 8 * GlobalSettings.ScreenWidthInch );
				if ( !changeinprogress )
				{
					numericUpDownOffsetPixels.Value = GlobalSettings.PositionVertical;
					numericUpDownOffsetPercentage.Value = (int)( 100 * GlobalSettings.PositionVertical / GlobalSettings.ScreenHeightPx );
				}
			}
			if ( GlobalSettings.Vertical )
			{
				if ( GlobalSettings.Inches )
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.ScreenHeightInch ) )
					{
						comboBoxScaleLength.Items.Insert( 0, GlobalSettings.ScreenHeightInch );
						comboBoxScaleLength.SelectedIndex = 0;
					}
				}
				else
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.SelectedHeightCm ) )
					{
						comboBoxScaleLength.Items.Insert( 0, GlobalSettings.SelectedHeightCm );
						comboBoxScaleLength.SelectedIndex = 0;
					}
				}
			}
			else
			{
				if ( GlobalSettings.Vertical )
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.ScreenWidthInch ) )
					{
						comboBoxScaleLength.Items.Insert( 0, GlobalSettings.ScreenWidthInch );
						comboBoxScaleLength.SelectedIndex = 0;
					}
				}
				else
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.SelectedWidthCm ) )
					{
						comboBoxScaleLength.Items.Insert( 0, GlobalSettings.SelectedWidthCm );
						comboBoxScaleLength.SelectedIndex = 0;
					}
				}
			}
			comboBoxUnits.SelectedIndex = ( GlobalSettings.Inches ? 1 : 0 );
			Application.DoEvents( );
		}


		#region Event Handlers

		private void FormConfig_Load( object sender, EventArgs e )
		{
			// move window to lower left corner
			this.Location = new Point( 0, Screen.PrimaryScreen.Bounds.Height - this.Height );
			// populate line color dropdown
			if ( comboBoxLineColor.Items.Count == 0 )
			{
				foreach ( string color in GlobalFunctions.ListColors( ) )
				{
					comboBoxLineColor.Items.Add( color );
				}
			}
			for ( int i = 0; i < comboBoxLineColor.Items.Count; i++ )
			{
				if ( comboBoxLineColor.Items[i].ToString( ) == GlobalSettings.PenColor.Name )
				{
					comboBoxLineColor.SelectedIndex = i;
				}
			}
			// populate background color dropdown
			if ( comboBoxBackgroundColor.Items.Count == 0 )
			{
				foreach ( string color in GlobalFunctions.ListColors( ) )
				{
					comboBoxBackgroundColor.Items.Add( color );
				}
			}
			for ( int i = 0; i < comboBoxBackgroundColor.Items.Count; i++ )
			{
				if ( comboBoxBackgroundColor.Items[i].ToString( ) == GlobalSettings.BackgroundColor.Name )
				{
					comboBoxBackgroundColor.SelectedIndex = i;
				}
			}
			// populate scale length dropdown
			{
				if ( GlobalSettings.Vertical )
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.LargestHeightCm ) )
					{
						comboBoxScaleLength.Items.Add( GlobalSettings.LargestHeightCm );
					}
				}
				else
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.LargestWidthCm ) )
					{
						comboBoxScaleLength.Items.Add( GlobalSettings.LargestWidthCm );
					}
				}
			}
			if ( Screen.AllScreens.Length > 1 )
			{
				if ( GlobalSettings.Vertical )
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.SmallestHeightCm ) )
					{
						comboBoxScaleLength.Items.Insert( 0, GlobalSettings.SmallestHeightCm );
					}
				}
				else
				{
					if ( !comboBoxScaleLength.Items.Contains( GlobalSettings.SmallestWidthCm ) )
					{
						comboBoxScaleLength.Items.Insert( 0, GlobalSettings.SmallestWidthCm );
					}
				}
			}
			Debug.Print( comboBoxScaleLength.Items.Count.ToString( ) + " Scale Length values" );
			foreach ( var item in comboBoxScaleLength.Items )
			{
				Debug.Print( item.ToString( ) );
			}
			UpdateFormConfig( );
		}


		private void FormConfig_KeyUp( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Escape )
			{
				this.Close( );
			}
			else if ( e.KeyCode == Keys.F1 )
			{
				GlobalFunctions.ShowHelp( );
			}
		}


		private void buttonReset_Click( object sender, EventArgs e )
		{
			if ( File.Exists( GlobalSettings.ConfigFile ) )
			{
				File.Delete( GlobalSettings.ConfigFile );
			}
			GlobalFunctions.SetDefaults( );
			UpdateFormConfig( );
			GlobalFunctions.RedrawScreen( );
			if ( !GlobalSettings.Dynamic )
			{
				FormProvider.Config.Close( );
			}
		}


		private void buttonSave_Click( object sender, EventArgs e )
		{
			GlobalFunctions.SaveConfig( );
			GlobalFunctions.RedrawScreen( );
			if ( !GlobalSettings.Dynamic )
			{
				FormProvider.Config.Close( );
			}
		}


		private void checkBoxOpaqueBackground_CheckedChanged( object sender, EventArgs e )
		{
			GlobalSettings.OpaqueBackground = checkBoxOpaqueBackground.Checked;
			comboBoxBackgroundColor.Enabled = checkBoxOpaqueBackground.Checked;
		}


		private void labelOpaqueBackground_Click( object sender, EventArgs e )
		{
			checkBoxOpaqueBackground.Checked = !checkBoxOpaqueBackground.Checked;
		}


		private void comboBoxBackgroundColor_SelectedIndexChanged( object sender, EventArgs e )
		{
			GlobalSettings.BackgroundColor = Color.FromName( comboBoxBackgroundColor.SelectedItem.ToString( ) );
		}


		private void comboBoxLineColor_SelectedIndexChanged( object sender, EventArgs e )
		{
			GlobalSettings.PenColor = Color.FromName( comboBoxLineColor.SelectedItem.ToString( ) );
			UpdateFormConfig( );
			if ( GlobalSettings.Dynamic )
			{
				GlobalFunctions.RedrawScreen( );
			}
		}


		private void comboBoxPosition_SelectedIndexChanged( object sender, EventArgs e )
		{
			GlobalSettings.Vertical = ( comboBoxPosition.SelectedItem.ToString( ) == "vertical" );
			UpdateFormConfig( );
			if ( GlobalSettings.Dynamic )
			{
				GlobalFunctions.RedrawScreen( );
			}
		}


		private void comboBoxScaleLength_SelectedIndexChanged( object sender, EventArgs e )
		{
			if ( GlobalSettings.Vertical )
			{
				GlobalSettings.SelectedHeightCm = int.Parse( comboBoxScaleLength.SelectedItem.ToString( ) );
			}
			else
			{
				GlobalSettings.SelectedWidthCm = int.Parse( comboBoxScaleLength.SelectedItem.ToString( ) );
			}
		}


		private void comboBoxUnits_SelectedIndexChanged( object sender, EventArgs e )
		{
			GlobalSettings.Inches = ( comboBoxUnits.SelectedItem.ToString( ) == "inch" );
			UpdateFormConfig( );
			if ( GlobalSettings.Dynamic )
			{
				GlobalFunctions.RedrawScreen( );
			}
		}


		private void numericUpDownOffsetPercentage_ValueChanged( object sender, EventArgs e )
		{
			if ( !changeinprogress )
			{
				changeinprogress = true;
				float percentage = float.Parse( numericUpDownOffsetPercentage.Value.ToString( ) );
				if ( GlobalSettings.Vertical )
				{
					GlobalSettings.PositionHorizontal = (int)( percentage * GlobalSettings.ScreenWidthPx / 100F );
					numericUpDownOffsetPixels.Value = GlobalSettings.PositionHorizontal;
				}
				else
				{
					GlobalSettings.PositionVertical = (int)( percentage * GlobalSettings.ScreenHeightPx / 100F );
					numericUpDownOffsetPixels.Value = GlobalSettings.PositionVertical;
				}
				UpdateFormConfig( );
				if ( GlobalSettings.Dynamic )
				{
					GlobalFunctions.RedrawScreen( );
				}
				changeinprogress = false;
			}
		}


		private void numericUpDownOffsetPixels_ValueChanged( object sender, EventArgs e )
		{
			if ( !changeinprogress )
			{
				changeinprogress = true;
				if ( GlobalSettings.Vertical )
				{
					GlobalSettings.PositionHorizontal = int.Parse( numericUpDownOffsetPixels.Value.ToString( ) ).LimitToRange( 0, GlobalSettings.ScreenHeightPx );
					numericUpDownOffsetPercentage.Value = 100 * GlobalSettings.PositionHorizontal / GlobalSettings.ScreenWidthPx;
				}
				else
				{
					GlobalSettings.PositionVertical = int.Parse( numericUpDownOffsetPixels.Value.ToString( ) ).LimitToRange( 0, GlobalSettings.ScreenWidthPx );
					numericUpDownOffsetPercentage.Value = 100 * GlobalSettings.PositionVertical / GlobalSettings.ScreenHeightPx;
				}
				UpdateFormConfig( );
				if ( GlobalSettings.Dynamic )
				{
					GlobalFunctions.RedrawScreen( );
				}
				changeinprogress = false;
			}
		}

		#endregion Event Handlers
	}
}
