﻿using System;
using System.Collections.Generic;
using System.Data;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Management;
using System.Reflection;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;


namespace Ruler
{
	public partial class FormRuler : Form
	{
		static bool interactive = false;
		static bool usesavedsettings = false;
		static bool configset = false;
		static bool showhelp = false;
		static string helptext = string.Empty;


		public FormRuler( )
		{
			InitializeComponent( );
		}


		public static void DrawRuler( PaintEventArgs e )
		{
			float division;
			float offset;
			int divider;
			int screensizecm;

			if ( GlobalSettings.Inches )
			{
				division = GlobalSettings.EighthOfAnInch;
				divider = 8;
			}
			else
			{
				division = GlobalSettings.Millimeter;
				divider = 10;
			}

			if ( GlobalSettings.Vertical )
			{
				if ( GlobalSettings.PositionHorizontal == -1 )
				{
					GlobalSettings.PositionHorizontal = GlobalSettings.HalfWidth;
				}
			}
			else
			{
				if ( GlobalSettings.PositionVertical == -1 )
				{
					GlobalSettings.PositionVertical = GlobalSettings.HalfHeight;
				}
			}

			if ( GlobalSettings.OpaqueBackground )
			{
				using ( Pen broadpen = new Pen( GlobalSettings.BackgroundColor, 29F ) )
				{
					if ( GlobalSettings.Vertical )
					{
						e.Graphics.DrawLine( broadpen, GlobalSettings.PositionHorizontal - 13, 0, GlobalSettings.PositionHorizontal - 13, GlobalSettings.ScreenHeightPx );
					}
					else
					{
						e.Graphics.DrawLine( broadpen, 0, GlobalSettings.PositionVertical + 13, GlobalSettings.ScreenWidthPx, GlobalSettings.PositionVertical + 13 );
					}

				}
			}

			using ( Pen pen = new Pen( GlobalSettings.PenColor ) )
			{
				if ( GlobalSettings.Vertical )
				{
					e.Graphics.DrawLine( pen, GlobalSettings.PositionHorizontal, 0, GlobalSettings.PositionHorizontal, GlobalSettings.ScreenHeightPx );
					screensizecm = GlobalSettings.SelectedHeightCm;
				}
				else
				{
					e.Graphics.DrawLine( pen, 0, GlobalSettings.PositionVertical, GlobalSettings.ScreenWidthPx, GlobalSettings.PositionVertical );
					screensizecm = GlobalSettings.SelectedWidthCm;
				}
				for ( int i = 0; i < divider * screensizecm; i++ )
				{
					offset = (float)( i * division );
					float length;
					if ( i % ( 10 * divider ) == 0 )
					{
						length = 25F;
					}
					else if ( i % divider == 0 )
					{
						length = 10F;
					}
					else if ( i % ( divider / 2 ) == 0 )
					{
						length = 6F;
					}
					else
					{
						length = 3F;
					}
					if ( GlobalSettings.Vertical )
					{
						e.Graphics.DrawLine( pen, (float)GlobalSettings.PositionHorizontal, offset, (float)GlobalSettings.PositionHorizontal - length, offset );
					}
					else
					{
						e.Graphics.DrawLine( pen, offset, (float)GlobalSettings.PositionVertical, offset, (float)GlobalSettings.PositionVertical + length );
					}
				}
			}
		}


		public void ParseCommandLine( )
		{
			List<string> args = Environment.GetCommandLineArgs( ).Skip( 1 ).ToList( );

			foreach ( string arg in args )
			{
				if ( arg.ToUpper( ) == "/S" )
				{
					usesavedsettings = true;
					GlobalFunctions.ReadConfig( );
				}
			}

			foreach ( string arg in args )
			{
				string key = arg.ToUpper( );
				string val = string.Empty;
				if ( arg.Contains( ":" ) )
				{
					key = arg.Split( ':' )[0].ToUpper( );
					val = arg.Split( ':' )[1];
				}
				switch ( key )
				{
					case "/?":
						showhelp = true;
						break;
					case "/C":
						if ( GlobalFunctions.ListColors( ).Contains( val, StringComparer.OrdinalIgnoreCase ) )
						{
							GlobalSettings.PenColor = Color.FromName( val );
						}
						break;
					case "/D":
						GlobalSettings.Dynamic = true;
						break;
					case "/I":
						interactive = true;
						break;
					case "/INCH":
						GlobalSettings.Inches = true;
						break;
					case "/M":
						switch ( val.ToUpper( ) )
						{
							case "MAX":
								GlobalSettings.SelectedHeightCm = GlobalSettings.LargestHeightCm;
								GlobalSettings.SelectedWidthCm = GlobalSettings.LargestWidthCm;
								break;
							case "MIN":
								GlobalSettings.SelectedHeightCm = GlobalSettings.SmallestHeightCm;
								GlobalSettings.SelectedWidthCm = GlobalSettings.SmallestWidthCm;
								break;
							default:
								if ( int.TryParse( arg, out int testwidth ) )
								{
									int testheight = (int)( (float)testwidth * (float)GlobalSettings.LargestHeightCm / (float)GlobalSettings.LargestWidthCm );
									GlobalSettings.SelectedHeightCm = testheight;
									GlobalSettings.SelectedWidthCm = testwidth;
								}
								else
								{
									GlobalSettings.SelectedHeightCm = GlobalSettings.LargestHeightCm;
									GlobalSettings.SelectedWidthCm = GlobalSettings.LargestWidthCm;
								}
								break;
						}
						break;
					case "/O":
						GlobalSettings.OpaqueBackground = true;
						if ( !string.IsNullOrEmpty( val ) )
						{
							GlobalSettings.BackgroundColor = Color.FromName( val );
						}
						break;
					case "/S":
						usesavedsettings = true;
						break;
					case "/V":
						GlobalSettings.Vertical = true;
						break;
					case "/X":
						if ( int.TryParse( val, out int test ) )
						{
							GlobalSettings.PositionHorizontal = test;
						}
						else
						{
							GlobalSettings.PositionHorizontal = -1;
						}
						break;
					case "/Y":
						if ( int.TryParse( val, out test ) )
						{
							GlobalSettings.PositionVertical = test;
						}
						else
						{
							GlobalSettings.PositionVertical = -1;
						}
						break;
					default:
						GlobalFunctions.ShowHelp( );
						break;
				}
			}

			GlobalSettings.SelectedHeightCm = GlobalSettings.SelectedHeightCm;
			//Debug.Print( "screen height = " + GlobalSettings.SelectedHeightCm.ToString( ) + "cm" );
			GlobalSettings.ScreenHeightInch = (float)GlobalSettings.SelectedHeightCm / 2.54F;
			GlobalSettings.SelectedWidthCm = GlobalSettings.SelectedWidthCm;
			//Debug.Print( "screen width = " + GlobalSettings.SelectedWidthCm.ToString( ) + "cm" );
			GlobalSettings.ScreenWidthInch = (float)GlobalSettings.SelectedWidthCm / 2.54F;
			GlobalSettings.ScreenHeightPx = Screen.PrimaryScreen.Bounds.Height;
			GlobalSettings.ScreenWidthPx = Screen.PrimaryScreen.Bounds.Width;
			GlobalSettings.HalfHeight = (int)( Screen.PrimaryScreen.Bounds.Height / 2 );
			GlobalSettings.HalfWidth = (int)( Screen.PrimaryScreen.Bounds.Width / 2 );
			if ( GlobalSettings.Vertical )
			{
				GlobalSettings.Millimeter = (float)GlobalSettings.ScreenHeightPx / ( 10 * GlobalSettings.SelectedHeightCm );
				GlobalSettings.EighthOfAnInch = (float)GlobalSettings.ScreenHeightPx / ( 8 * GlobalSettings.ScreenHeightInch );
			}
			else
			{
				GlobalSettings.Millimeter = (float)GlobalSettings.ScreenWidthPx / ( 10 * GlobalSettings.SelectedWidthCm );
				GlobalSettings.EighthOfAnInch = (float)GlobalSettings.ScreenWidthPx / ( 8 * GlobalSettings.ScreenWidthInch );
			}
			//Debug.Print( "1mm = " + GlobalSettings.Millimeter.ToString( ) + " px" );

			if ( usesavedsettings )
			{
				GlobalFunctions.ReadConfig( );
			}

			if ( interactive )
			{
				GlobalFunctions.SetConfig( );
			}

			if ( GlobalSettings.Dynamic )
			{
				usesavedsettings = true;
			}

			if ( showhelp )
			{
				GlobalFunctions.ShowHelp( );
			}
		}


		[DllImport( "kernel32.dll" )]
		static extern bool AttachConsole( int dwProcessId );
		private const int ATTACH_PARENT_PROCESS = -1;


		#region Event Handlers

		private void FormRuler_Paint( object sender, PaintEventArgs e )
		{
			if ( !configset )
			{
				GlobalFunctions.GetMonitorDimensions( );
				GlobalFunctions.SetDefaults( );
				ParseCommandLine( );
				configset = true;
			}
			DrawRuler( e );
		}


		private void FormRuler_FormClosing( object sender, FormClosingEventArgs e )
		{
			if ( showhelp )
			{
				string helptext = GlobalSettings.HelpText;
				helptext = helptext.Replace( "console-output-\n        \t\t\t\tfrom-winforms-application", "console-output-from-winforms-application" );
				Console.Error.WriteLine( helptext );
			}
		}


		private void FormRuler_KeyUp( object sender, KeyEventArgs e )
		{
			if ( e.KeyCode == Keys.Escape )
			{
				Application.Exit( );
			}
			else if ( e.KeyCode == Keys.F1 )
			{
				GlobalFunctions.ShowHelp( );
			}
			else if ( e.KeyCode == Keys.F2 )
			{
				GlobalFunctions.SetConfig( );
			}
		}


		private void FormRuler_Load( object sender, EventArgs e )
		{
			// https://www.csharp411.com/console-output-from-winforms-application/
			// redirect console output to parent process;
			// must be before any calls to Console.WriteLine( )
			AttachConsole( ATTACH_PARENT_PROCESS );
		}

		#endregion Event Handlers
	}


	public class Defaults
	{
		public static Color BackgroundColor
		{
			get
			{
				return Color.White;
			}
		}


		public static bool Dynamic
		{
			get
			{
				return false;
			}
		}


		public static bool Inches
		{
			get
			{
				return false;
			}
		}


		public static int[] MonitorSize
		{
			get
			{
				return new int[] { 0, 0 };
			}
		}


		public static bool OpaqueBackground
		{
			get
			{
				return false;
			}
		}


		public static Color PenColor
		{
			get
			{
				return Color.Black;
			}
		}


		public static int PositionHorizontal
		{
			get
			{
				return -1;
			}
		}


		public static int PositionVertical
		{
			get
			{
				return -1;
			}
		}


		public static bool Vertical
		{
			get
			{
				return false;
			}
		}
	}


	public class GlobalFunctions
	{
		public static string GetConfigFile( )
		{
			string exefile = new System.Uri( Assembly.GetExecutingAssembly( ).CodeBase ).LocalPath;
			string parentfolder = Path.GetDirectoryName( exefile );
			string filename = Path.GetFileNameWithoutExtension( exefile );
			string configfile = Path.Combine( parentfolder, filename + ".ini" );
			return configfile;
		}


		public static void GetMonitorDimensions( )
		{
			using ( ManagementObjectSearcher searcher = new ManagementObjectSearcher( @"root\WMI", "SELECT MaxHorizontalImageSize,MaxVerticalImageSize FROM WmiMonitorBasicDisplayParams" ) )
			{
				Debug.Print( "GetMonitorDimensions( )" );
				ManagementObjectCollection colItems = searcher.Get( );
				Dictionary<int, int> monitors = new Dictionary<int, int>( );
				int horsmallest = 200;
				int horlargest = 0;
				int versmallest = 200;
				int verlargest = 0;
				foreach ( ManagementObject queryObj in colItems.Cast<ManagementObject>( ) )
				{
					int hor = int.Parse( queryObj["MaxHorizontalImageSize"].ToString( ) );
					int ver = int.Parse( queryObj["MaxVerticalImageSize"].ToString( ) );
					monitors.Add( hor, ver );
					horsmallest = Math.Min( horsmallest, hor );
					versmallest = Math.Min( versmallest, ver );
					horlargest = Math.Max( horlargest, hor );
					verlargest = Math.Max( verlargest, ver );
				}
				GlobalSettings.LargestHeightCm = verlargest;
				GlobalSettings.LargestWidthCm = horlargest;
				GlobalSettings.SmallestHeightCm = versmallest;
				GlobalSettings.SmallestWidthCm = horsmallest;
			}
		}


		public static List<string> ListColors( )
		{
			List<string> colors = Enum.GetNames( typeof( ConsoleColor ) ).ToList( );
			colors.Remove( "DarkYellow" );
			colors.Remove( "Magenta" );
			colors.Add( "Purple" );
			colors.Add( "Orange" );
			colors.Sort( );
			return colors;
		}


		public static void ReadConfig( )
		{
			Debug.Print( "ReadConfig( )" );
			if ( string.IsNullOrEmpty( GlobalSettings.ConfigFile ) )
			{
				GlobalSettings.ConfigFile = GetConfigFile( );
			}
			if ( File.Exists( GlobalSettings.ConfigFile ) )
			{
				string config = File.ReadAllText( GlobalSettings.ConfigFile, Encoding.ASCII );
				string[] lines = config.Split( "\n\r".ToCharArray( ), StringSplitOptions.RemoveEmptyEntries );
				foreach ( string line in lines )
				{
					if ( line.Contains( "=" ) )
					{
						string key = line.Split( '=' )[0];
						string val = line.Split( '=' )[1];
						int test;
						switch ( key )
						{
							case "BackgroundColor":
								if ( !string.IsNullOrEmpty( val ) && ListColors( ).Contains( val, StringComparer.OrdinalIgnoreCase ) )
								{
									GlobalSettings.BackgroundColor = Color.FromName( val );
								}
								else
								{
									GlobalSettings.BackgroundColor = Defaults.BackgroundColor;
								}
								break;
							case "DynamicUpdate":
								GlobalSettings.Dynamic = ( val == "1" );
								break;
							case "OpaqueBackground":
								GlobalSettings.OpaqueBackground = ( val == "1" );
								break;
							case "PenColor":
								if ( ListColors( ).Contains( val, StringComparer.OrdinalIgnoreCase ) )
								{
									GlobalSettings.PenColor = Color.FromName( val );
								}
								else
								{
									GlobalSettings.PenColor = Defaults.PenColor;
								}
								break;
							case "PositionHorizontal":
								if ( int.TryParse( val, out test ) )
								{
									GlobalSettings.PositionHorizontal = test.LimitToRange( 0, Screen.PrimaryScreen.Bounds.Height );
								}
								else
								{
									GlobalSettings.PositionHorizontal = GlobalSettings.HalfHeight;
								}
								break;
							case "PositionVertical":
								if ( int.TryParse( val, out test ) )
								{
									GlobalSettings.PositionVertical = test.LimitToRange( 0, Screen.PrimaryScreen.Bounds.Width );
								}
								else
								{
									GlobalSettings.PositionVertical = GlobalSettings.HalfWidth;
								}
								break;
							case "ScaleInInches":
								GlobalSettings.Inches = ( val == "1" );
								break;
							case "ScreenHeight":
								if ( int.TryParse( val, out test ) )
								{
									GlobalSettings.SelectedHeightCm = test.LimitToRange( 10, 200 );
								}
								else
								{
									GlobalSettings.SelectedHeightCm = GlobalSettings.LargestHeightCm;
								}
								break;
							case "ScreenWidth":
								if ( int.TryParse( val, out test ) )
								{
									GlobalSettings.SelectedWidthCm = test.LimitToRange( 10, 200 );
								}
								else
								{
									GlobalSettings.SelectedWidthCm = GlobalSettings.LargestWidthCm;
								}
								break;
							case "VerticalRuler":
								GlobalSettings.Vertical = ( val == "1" );
								break;
						}
					}
				}
			}
			else
			{
				SetDefaults( );
				SaveConfig( );
			}
		}


		#region RedrawScreen Code

		// Code to redraw the screen by RogerN on StackOverflow.com
		// https://stackoverflow.com/a/16066507
		const int RDW_INVALIDATE = 0x0001;
		const int RDW_ALLCHILDREN = 0x0080;
		const int RDW_UPDATENOW = 0x0100;

		[DllImport( "User32.dll" )]
		static extern bool RedrawWindow( IntPtr hwnd, IntPtr rcUpdate, IntPtr regionUpdate, int flags );

		#endregion RedrawScreen Code


		public static void RedrawScreen( )
		{
			// Code to redraw the screen by RogerN on StackOverflow.com
			// https://stackoverflow.com/a/16066507
			RedrawWindow( IntPtr.Zero, IntPtr.Zero, IntPtr.Zero, RDW_INVALIDATE | RDW_ALLCHILDREN | RDW_UPDATENOW );
		}


		public static void SaveConfig( )
		{
			Debug.Print( "SaveConfig( )" );
			if ( string.IsNullOrEmpty( GlobalSettings.ConfigFile ) )
			{
				GlobalSettings.ConfigFile = GetConfigFile( );
			}
			string config = "[RulerSettings]\n";
			config += string.Format( "BackgroundColor={0}\n", GlobalSettings.BackgroundColor.Name );
			config += string.Format( "ScaleInInches={0}\n", ( GlobalSettings.Inches ? "1" : "0" ) );
			config += string.Format( "DynamicUpdate={0}\n", ( GlobalSettings.Dynamic ? "1" : "0" ) );
			config += string.Format( "OpaqueBackground={0}\n", ( GlobalSettings.OpaqueBackground ? "1" : "0" ) );
			config += string.Format( "PenColor={0}\n", GlobalSettings.PenColor.Name );
			config += string.Format( "PositionHorizontal={0}\n", GlobalSettings.PositionHorizontal );
			config += string.Format( "PositionVertical={0}\n", GlobalSettings.PositionVertical );
			config += string.Format( "ScreenHeight={0}\n", GlobalSettings.SelectedHeightCm );
			config += string.Format( "ScreenWidth={0}\n", GlobalSettings.SelectedWidthCm );
			config += string.Format( "VerticalRuler={0}\n", ( GlobalSettings.Vertical ? "1" : "0" ) );
			File.WriteAllText( GlobalSettings.ConfigFile, config, Encoding.ASCII );
		}


		public static void SetConfig( )
		{
			Debug.Print( "SetConfig( )" );
			FormProvider.Config.WindowState = FormWindowState.Normal;
			FormProvider.Config.Show( );
		}


		public static void SetDefaults( )
		{
			Debug.Print( "SetDefaults( )" );
			GlobalSettings.BackgroundColor = Defaults.BackgroundColor;
			GlobalSettings.Dynamic = Defaults.Dynamic;
			GlobalSettings.PenColor = Defaults.PenColor;
			GlobalSettings.SelectedHeightCm = GlobalSettings.LargestHeightCm;
			GlobalSettings.SelectedWidthCm = GlobalSettings.LargestWidthCm;
			GlobalSettings.Inches = Defaults.Inches;
			GlobalSettings.HalfHeight = Screen.PrimaryScreen.Bounds.Height / 2;
			GlobalSettings.Millimeter = (float)GlobalSettings.ScreenWidthPx / ( 10 * GlobalSettings.SelectedWidthCm );
			GlobalSettings.OpaqueBackground = Defaults.OpaqueBackground;
			GlobalSettings.PositionVertical = GlobalSettings.HalfHeight;
			GlobalSettings.ScreenHeightPx = Screen.PrimaryScreen.Bounds.Height;
			GlobalSettings.SelectedWidthCm = GlobalSettings.SelectedWidthCm;
			GlobalSettings.ScreenWidthInch = (float)GlobalSettings.SelectedWidthCm / 2.54F;
			GlobalSettings.ScreenWidthPx = Screen.PrimaryScreen.Bounds.Width;
			GlobalSettings.Vertical = Defaults.Vertical;
		}


		public static void ShowHelp( )
		{
			string helptext = GlobalSettings.HelpText;
			string title = "Help for " + helptext.Split( '\n' )[0];
			MessageBox.Show( helptext, title, MessageBoxButtons.OKCancel, MessageBoxIcon.None );
		}
	}


	public class GlobalSettings
	{
		private static Color _BackgroundColor = Defaults.BackgroundColor;
		public static Color BackgroundColor
		{
			get
			{
				return _BackgroundColor;
			}
			set
			{
				_BackgroundColor = value;
			}
		}


		public static string ConfigFile { get; set; }


		private static bool _Dynamic = Defaults.Dynamic;
		public static bool Dynamic
		{
			get
			{
				return _Dynamic;
			}
			set
			{
				_Dynamic = value;
			}
		}


		public static float EighthOfAnInch { get; set; }


		private static int _HalfHeight;
		public static int HalfHeight
		{
			get
			{
				return _HalfHeight;
			}
			set
			{
				_HalfHeight = value;
			}
		}


		private static int _HalfWidth;
		public static int HalfWidth
		{
			get
			{
				return _HalfWidth;
			}
			set
			{
				_HalfWidth = value;
			}
		}


		public static string HelpText
		{
			get
			{
				string helptext = "Ruler,  Version " + Assembly.GetExecutingAssembly( ).GetName( ).Version.ToString( );
				helptext += "\nShow a ruler on the primary monitor\n\n";
				helptext += "Usage:  \tRuler.exe  [ options ]\n\n";
				helptext += "Options:\t/?    \t show this help text\n";
				helptext += "        \t/C:color\t set line Color (default: black)\n";
				helptext += "        \t/D    \t keep settings dialog open after saving changes\n";
				helptext += "        \t      \t (requires /I and/or pressing F2)\n";
				helptext += "        \t/I    \t Interactive configuration\n";
				helptext += "        \t/INCH \t use Inches instead of centimeters for scale\n";
				helptext += "        \t/M:mon\t select monitor size:\n";
				helptext += "        \t      \t \"max\" for largest or \"min\" for smallest\n";
				helptext += "        \t      \t (if multiple monitors), or monitor width in cm\n";
				helptext += "        \t      \t (default: randomly selected monitor's width)\n";
				helptext += "        \t/O[:color] show Opaque background lint behind ruler\n";
				helptext += "        \t      \t (default color: white)\n";
				helptext += "        \t/S    \t use previously Saved configuration\n";
				helptext += "        \t/V    \t show Vertical ruler (default: horizontal)\n";
				helptext += "        \t/Y:yyy\t vertical (Y) position in pixels\n";
				helptext += "        \t      \t (default: half of screen height)\n";
				helptext += "        \t/X:xxx\t horizontal (X) position in pixels\n";
				helptext += "        \t      \t (default: half of screen width)\n\n";
				helptext += "Remarks:\tPress Esc to abort the program, F1 for help, or F2 for settings.\n";
				helptext += "        \tMonitor selection is only intended for accurate scale,\n";
				helptext += "        \tbut the ruler will always be shown on the primary monitor.\n";
				helptext += "        \t/Y cannot be combined with /V, whereas /X requires /V\n\n";
				helptext += "Credits:\tCode to redraw the screen by RogerN on StackOverflow.com\n";
				helptext += "        \thttps://stackoverflow.com/a/16066507\n";
				helptext += "        \tSingleton settings form by djdd87 on StackOverflow.com\n";
				helptext += "        \thttps://stackoverflow.com/a/3005761\n";
				helptext += "        \tSet a property and then make it readonly by Tomer-Barkan\n";
				helptext += "        \thttps://discussions.unity.com/t/89395/2\n";
				helptext += "        \tConsole output from a WinForms program by Timm\n";
				helptext += "        \thttps://www.csharp411.com/console-output-\n";
				helptext += "        \t\t\t\tfrom-winforms-application/\n\n";
				helptext += "Written by Rob van der Woude\n";
				helptext += "https://www.robvanderwoude.com";
				return helptext;
			}
		}


		private static bool _Inches = Defaults.Inches;
		public static bool Inches
		{
			get
			{
				return _Inches;
			}
			set
			{
				_Inches = value;
			}
		}


		// Code to set a property once, and then make
		// it readonly, by Tomer-Barkan on Unity.com
		// https://discussions.unity.com/t/89395/2
		private static int _LargestHeightCm = -1;
		public static int LargestHeightCm
		{
			get
			{
				return _LargestHeightCm;
			}
			set
			{
				if ( _LargestHeightCm == -1 )
				{
					_LargestHeightCm = value;
				}
			}
		}


		private static int _LargestWidthCm = -1;
		public static int LargestWidthCm
		{
			get
			{
				return _LargestWidthCm;
			}
			set
			{
				if ( _LargestWidthCm == -1 )
				{
					_LargestWidthCm = value;
				}
			}
		}


		public static float Millimeter { get; set; }


		private static bool _OpaqueBackground = Defaults.OpaqueBackground;
		public static bool OpaqueBackground
		{
			get
			{
				return _OpaqueBackground;
			}
			set
			{
				_OpaqueBackground = value;
			}
		}


		private static Color _PenColor = Defaults.PenColor;
		public static Color PenColor
		{
			get
			{
				return _PenColor;
			}
			set
			{
				_PenColor = value;
			}
		}


		private static int _PositionHorizontal = Defaults.PositionHorizontal;
		public static int PositionHorizontal
		{
			get
			{
				return _PositionHorizontal;
			}
			set
			{
				_PositionHorizontal = value.LimitToRange( 0, Screen.PrimaryScreen.Bounds.Width );
			}
		}


		private static int _PositionVertical = Defaults.PositionVertical;
		public static int PositionVertical
		{
			get
			{
				return _PositionVertical;
			}
			set
			{
				_PositionVertical = value.LimitToRange( 0, Screen.PrimaryScreen.Bounds.Height );
			}
		}

		
		public static float ScreenHeightInch { get; set; }


		public static int ScreenHeightPx { get; set; }


		public static float ScreenWidthInch { get; set; }


		public static int ScreenWidthPx { get; set; }


		private static int _SelectedHeightCm;
		public static int SelectedHeightCm
		{
			get
			{
				return _SelectedHeightCm;
			}
			set
			{
				_SelectedHeightCm = value;
			}
		}


		private static int _SelectedWidthCm;
		public static int SelectedWidthCm
		{
			get
			{
				return _SelectedWidthCm;
			}
			set
			{
				_SelectedWidthCm = value;
			}
		}


		private static int _SmallestHeightCm = -1;
		public static int SmallestHeightCm
		{
			get
			{
				return _SmallestHeightCm;
			}
			set
			{
				if ( _SmallestHeightCm == -1 )
				{
					_SmallestHeightCm = value;
				}
			}
		}


		private static int _SmallestWidthCm = -1;
		public static int SmallestWidthCm
		{
			get
			{
				return _SmallestWidthCm;
			}
			set
			{
				if ( _SmallestWidthCm == -1 )
				{
					_SmallestWidthCm = value;
				}
			}
		}


		private static bool _Vertical = Defaults.Vertical;
		public static bool Vertical
		{
			get
			{
				return _Vertical;
			}
			set
			{
				_Vertical = value;
			}
		}
	}


	public static class ExtensionMethods
	{
		// Extension by "dtb" on StackOverflow.com:
		// https://stackoverflow.com/a/3176628
		public static int LimitToRange( this int value, int inclusiveMinimum, int inclusiveMaximum )
		{
			if ( value < inclusiveMinimum )
			{
				return inclusiveMinimum;
			}
			if ( value > inclusiveMaximum )
			{
				return inclusiveMaximum;
			}
			return value;
		}


		public static float LimitToRange( this float value, float inclusiveMinimum, float inclusiveMaximum )
		{
			if ( value < inclusiveMinimum )
			{
				return inclusiveMinimum;
			}
			if ( value > inclusiveMaximum )
			{
				return inclusiveMaximum;
			}
			return value;
		}
	}


	public class FormProvider
	{
		// Singleton Pattern by djdd87 on StackOverflow.com
		// https://stackoverflow.com/a/3005761

		private static FormConfig _Config;
		public static FormConfig Config
		{
		get
		{
			if ( _Config == null || _Config.IsDisposed )
			{
				_Config = new FormConfig( );
			}
			return _Config;
			}
		}
	}
}
